import os
import pandas as pd


def dt_handle(date):
    try:
        return pd.to_datetime(date).strftime('%Y%m%d')
    except ValueError:
        return date


def set(country=None):
    """
    country: list or iterable
    """
    if country is not None:
        if isinstance(country, str):
            country = [
                country,
            ]
        os.environ['sangreal_calendar'] = ','.join(country)


"""
数据引擎
"""

import os

import yaml
from sangreal_db import DataBase
from sqlalchemy import create_engine

"""""" """""" """""" """""" """""" ""
"        读取配置文件内容         "
"""""" """""" """""" """""" """""" ""
CONFIG_FILE_NAME = "wind.yaml"
HOME_PATH = os.path.expanduser(f'~{os.sep}.sangreal{os.sep}wind')
if not os.path.exists(HOME_PATH):
    os.makedirs(HOME_PATH)
CONFIG_FILE = os.path.join(HOME_PATH, CONFIG_FILE_NAME)

YAML_TYPE = f"""
wind.config:
    engine: engine url
    schema: blank or other

bundle.config:
    dir: blank or other
"""

if not os.path.isfile(CONFIG_FILE):
#     print(f"{CONFIG_FILE_NAME} does not exist!, you can check {HOME_PATH} and touch it!\
#  The yaml' type is like {YAML_TYPE}")
    config = None
else:
    # 读取数据
    with open(CONFIG_FILE, 'r') as f:
        config = yaml.load(f, Loader=yaml.FullLoader)


def get_db(config, k):
    if config is not None:
        db_config = config.get(k, None)
        if db_config is None:
            raise ValueError(f"Please check the {CONFIG_FILE} and add {k}!\
    The yaml' type is like {YAML_TYPE}")
        engine = create_engine(db_config['engine'])
        schema = db_config['schema']
        return DataBase(engine, schema), engine
    else:
        return DataBase(None), None


def get_bundle(config, k):
    if config is not None:
        bundle_config = config.get(k, None)
        if bundle_config is not None:
            #         raise ValueError(f"Please check the {CONFIG_FILE} and add {k}!\
            # The yaml' type is like {YAML_TYPE}")
            bundle_dir = bundle_config['dir']
            if bundle_dir is None:
                bundle_dir = f"{os.path.expanduser(f'~{os.sep}.sangreal{os.sep}backtest{os.sep}bundle{os.sep}')}"
            return bundle_dir
        else:
            return
    else:
        return


# 实例化数据库 ENGINE
WIND_DB, ENGINE = get_db(config, 'wind.config')

# 回测框架本地文件存放的数据
BUNDLE_DIR = get_bundle(config, 'bundle.config')
if BUNDLE_DIR is not None:
    if not BUNDLE_DIR.endswith(os.sep):
        raise ValueError(f'bundle_dir的路径必须以{os.sep}结尾')
    if not os.path.exists(BUNDLE_DIR):
        os.makedirs(BUNDLE_DIR)



import pandas as pd
import datetime as dt
from fastcache import lru_cache
from sangreal_wind.utils.engines import WIND_DB
from sangreal_wind.sangreal_calendar import adjust_trade_dt


@lru_cache()
def get_index_weight_all(index):
    index = '399300.SZ' if index == '000300.SH' else index
    table = getattr(WIND_DB, 'AIndexHS300FreeWeight'.upper())
    df = WIND_DB.query(
        table.S_CON_WINDCODE, table.I_WEIGHT,
        table.TRADE_DT).filter(table.S_INFO_WINDCODE == index).order_by(
            table.TRADE_DT.desc()).to_df()
    df.columns = ['sid', 'weight', 'trade_dt']
    df.weight = df.weight / 100.0
    return df


def get_index_weight(index, trade_dt=None):
    """[获取指数成份权重]

    Arguments:
        index {[str]} -- [windcode of index]

    Keyword Arguments:
        trade_dt {[str or datetime]} -- [trade_dt] (default: {None})

    Returns:
        [pd.DataFrame] -- [sid, weight]
    """

    if trade_dt is None:
        trade_dt = dt.date.today()
    trade_dt = adjust_trade_dt(trade_dt)
    df = get_index_weight_all(index).copy()
    df = df[(df['trade_dt'] <= trade_dt)]
    if df.empty:
        return pd.DataFrame()

    # 取出最近一个交易日
    t = df.trade_dt.iloc[0]
    df = df[df['trade_dt'] == t]
    df.drop(['trade_dt'], axis=1, inplace=True)
    if t != trade_dt:
        table = getattr(WIND_DB, 'AShareEODPrices'.upper())
        q = WIND_DB.query(table.S_INFO_WINDCODE.label(
            'sid'), table.S_DQ_ADJCLOSE).filter(table.S_INFO_WINDCODE.in_(df.sid))
        c1 = q.filter(table.TRADE_DT == t).to_df().set_index('sid').iloc[:, 0]
        c2 = q.filter(table.TRADE_DT == trade_dt).to_df(
        ).set_index('sid').iloc[:, 0]
        adjust_factor = c2 / c1
        df.set_index('sid', inplace=True)
        df['weight'] = df['weight'] * adjust_factor
        df['weight'] /= df['weight'].sum()
        df.reset_index(inplace=True)

    return df.reset_index(drop=True)


if __name__ == '__main__':
    df = get_index_weight('000300.SH')
    print(df.head())
